/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Quickfile extension for Thunderbird.
 *
 * The Initial Developer of the Original Code is Paul Tomlin.
 * Portions created by the Initial Developer are Copyright (C) 2005, 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Paul Tomlin
 *  Mark Shuttleworth
 *
 * ***** END LICENSE BLOCK ***** */

/* $Id: optionsGeneral.js,v 1.3 2006/11/20 23:41:01 paultomlin Exp $ */

var gGeneralPane =
{
  onPaneLoad: function _onPaneLoad ()
  {
    document.getElementById('quickfile.search.root.account.radio').value =
      document.getElementById('quickfile.search.root.account.menulist').value;

    this.updateSearchRootSearchRoot();

    var searchRootFolder;
    var searchRootUri = document.getElementById('quickfile.search.root').value;
    switch (searchRootUri) {
      case QF_SEARCHROOT_ALL:
      case QF_SEARCHROOT_ACCOUNT:
        // handled statically
        break;
      default:
        searchRootFolder = QuickfileCommon.getFolderFromURI(searchRootUri);
        if (searchRootFolder) {
          if (searchRootFolder.isServer) {
            document.getElementById('quickfile.search.root.account.radio').value = searchRootUri;
            document.getElementById('quickfile.search.root.account.menulist').value = searchRootUri;
            document.getElementById('quickfile.search.root.radiogroup').selectedItem =
              document.getElementById('quickfile.search.root.account.radio');
          }
          else {
            var name = QuickfileCommon.getFolderRelativeName(searchRootFolder, null);
            document.getElementById('quickfile.search.root.folder.radio').value = searchRootUri;
            document.getElementById('quickfile.search.root.textbox').value = name;
            document.getElementById('quickfile.search.root.radiogroup').selectedItem =
              document.getElementById('quickfile.search.root.folder.radio');
          }
        }
        else {
          document.getElementById('quickfile.search.root').value = QF_SEARCHROOT_ALL;
          searchRootUri = QF_SEARCHROOT_ALL;
        }
        break;
    }

    this.updateDefaultFolder();
  },

  onSearchRootAccountMenulistCommand: function _onSearchRootAccountMenulistCommand ()
  {
    var uri = document.getElementById('quickfile.search.root.account.menulist').selectedItem.value;

    document.getElementById('quickfile.search.root.account.radio').value = uri;
    document.getElementById('quickfile.search.root').value = uri;

    document.getElementById('quickfile.search.root.radiogroup').selectedItem =
      document.getElementById('quickfile.search.root.account.radio');
  },

  onDefaultFolderEnableChange: function _onDefaultFolderEnableChange ()
  {
    document.getElementById('quickfile.defaultfolder.textbox').disabled =
      ! document.getElementById('quickfile.defaultfolder.enable').value;
  },

  onDefaultFolderTextInput: function _onDefaultFolderTextInput ()
  {
    // the nsIAutoCompleteSession is shared, apparently
    this.updateDefaultFolderSearchRoot();
  },

  onDefaultFolderTextEntered: function _onDefaultFolderTextEntered ()
  {
    document.getElementById('quickfile.defaultfolder.uri').value =
      document.getElementById('quickfile.defaultfolder.textbox').uriValue;
  },

  onSearchRootTextInput: function _onSearchRootTextInput ()
  {
    // the nsIAutoCompleteSession is shared, apparently
    this.updateSearchRootSearchRoot();
  },

  onSearchRootTextEntered: function _onSearchRootTextEntered ()
  {
    var uri = document.getElementById('quickfile.search.root.textbox').uriValue;

    document.getElementById('quickfile.search.root.folder.radio').value = uri;
    document.getElementById('quickfile.search.root').value = uri;

    document.getElementById('quickfile.search.root.radiogroup').selectedItem =
      document.getElementById('quickfile.search.root.folder.radio');
  },

  onSearchRootChange: function _onSearchRootChange()
  {
    this.updateDefaultFolder();
  },

  enforceDefaultFolderValidity: function _enforceDefaultFolderValidity ()
  {
    var enabled = document.getElementById('quickfile.defaultfolder.enable').value && this.isDefaultFolderValid();
    if (! enabled)
      document.getElementById('quickfile.defaultfolder.textbox').value = '';

    document.getElementById('quickfile.defaultfolder.enable').value = enabled;

    // sync default folder textbox enabled state
    this.onDefaultFolderEnableChange();
  },

  isDefaultFolderValid: function _isDefaultFolderValid ()
  {
    var defaultFolderUri = document.getElementById('quickfile.defaultfolder.uri').value;
    var searchRootUri = document.getElementById('quickfile.search.root').value;

    if (! defaultFolderUri)
      return false;

    if (searchRootUri == QF_SEARCHROOT_ACCOUNT)
      return false;

    var defaultFolder = QuickfileCommon.getFolderFromURI(defaultFolderUri);
    if (! defaultFolder)
      return false;

    if (searchRootUri == QF_SEARCHROOT_ALL)
      return true;

    var searchRootFolder = QuickfileCommon.getFolderFromURI(searchRootUri);
    if (! searchRootFolder)
      return false;

    if (! searchRootFolder.isAncestorOf(defaultFolder))
      return false;

    return true;
  },

  updateDefaultFolderSearchRoot: function _updateDefaultFolderSearchRoot ()
  {
    var searchRootUri = document.getElementById('quickfile.search.root').value;
    if (searchRootUri == QF_SEARCHROOT_ACCOUNT)
      return;

    var autoCompleteSession = document.getElementById('quickfile.defaultfolder.textbox').getSession(0);
    if (autoCompleteSession && typeof autoCompleteSession.QueryInterface == 'function') {
      autoCompleteSession.QueryInterface(nsICategoryManager)
                         .addCategoryEntry('quickfile', 'searchRoot', searchRootUri, false, true);
    }
  },

  updateDefaultFolderText: function _updateDefaultFolderText ()
  {
    this.updateDefaultFolderSearchRoot();

    var el = document.getElementById('quickfile.defaultfolder.textbox');

    var searchRootUri = document.getElementById('quickfile.search.root').value;
    if (searchRootUri == QF_SEARCHROOT_ACCOUNT) {
      el.value = '';
      return;
    }

    var searchRootFolder = null;
    if (searchRootUri != QF_SEARCHROOT_ALL)
      searchRootFolder = QuickfileCommon.getFolderFromURI(searchRootUri);
    
    var defaultFolderUri = document.getElementById('quickfile.defaultfolder.uri').value;
    var defaultFolder = QuickfileCommon.getFolderFromURI(defaultFolderUri);
    var name = QuickfileCommon.getFolderRelativeName(defaultFolder, searchRootFolder);
    el.value = name;
  },

  updateDefaultFolder: function _updateDefaultFolder ()
  {
    if (! this.isDefaultFolderValid()) {
      this.enforceDefaultFolderValidity();
      return;
    }
    
    this.updateDefaultFolderText();

    // sync default folder textbox enabled state
    this.onDefaultFolderEnableChange();
  },

  updateSearchRootSearchRoot: function _updateSearchRootSearchRoot()
  {
    var autoCompleteSession = document.getElementById('quickfile.search.root.textbox').getSession(0);
    if (autoCompleteSession && typeof autoCompleteSession.QueryInterface == 'function') {
      autoCompleteSession.QueryInterface(nsICategoryManager)
                         .addCategoryEntry('quickfile', 'searchRoot', QF_SEARCHROOT_ALL, false, true);
    }
  }
};
